import { Positions } from '../types';
/**
 * Calculates adjusted top positions for comments to prevent overlaps in the UI.
 *
 * **Description:**
 * This function ensures that comments are displayed without overlapping by adjusting their
 * top positions based on their desired locations and heights. It prioritizes the active
 * comment, keeping it at its desired position while minimally adjusting surrounding comments
 * to maintain readability and proximity to their associated text.
 *
 * **Parameters:**
 * - `textPositions: Record<string, { top: number }>`: Maps each comment ID to its desired top position.
 * - `activeCommentId: string | null`: The ID of the currently active (focused) comment, or `null` if none.
 * - `visibleComments: Set<string>`: A set of comment IDs that are currently visible in the UI.
 * - `commentHeights: Record<string, number>`: Maps each comment ID to its height in pixels.
 *
 * **Returns:**
 * - `Positions`: An object mapping each visible comment ID to its adjusted top position.
 *
 * **Behavior:**
 * - **Without an Active Comment:**
 *   - Sorts comments by their desired top positions.
 *   - Iterates from top to bottom, adjusting each comment downward only if it overlaps with the previous one.
 *
 * - **With an Active Comment:**
 *   - Keeps the active comment at its desired position.
 *   - Adjusts comments above the active comment upwards as necessary, prioritizing higher comments to minimize shifts.
 *   - Adjusts comments below the active comment downward to prevent overlaps.
 *
 * **Key Considerations:**
 * - Ensures minimal movement of comments to maintain their association with the text.
 * - Prevents comments from moving above a defined minimum top limit (e.g., 0px).
 * - Handles multiple overlapping comments by iteratively adjusting positions to resolve conflicts.
 *
 * **Usage:**
 * This function is essential for rendering a clean and user-friendly comments section,
 * especially in applications that mimic collaborative tools like Google Docs.
 */
export declare const calculatePositions: (textPositions: Record<string, {
    top: number;
}>, activeCommentId: string | null, visibleComments: Set<string>, commentHeights: Record<string, number>) => Positions;
